package models

import (
	"errors"
	"eta/eta_index_lib/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"reflect"
	"strconv"
	"strings"
	"time"
)

// Ljzzj 累计值转季度
type Ljzzj struct {
}

// Add 添加
func (obj Ljzzj) Add(params AddCalculateBatchParams) (edbInfo *EdbInfo, err error, errMsg string) {
	req := params.Req
	fromEdbInfo := params.FromEdbInfo
	edbCode := params.EdbCode
	if fromEdbInfo.Frequency != "季度" {
		errMsg = "请选择季度指标"
		err = errors.New(errMsg)
		return
	}
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
			fmt.Println(reflect.TypeOf(obj).Name(), "；Add,Err:"+err.Error())
		} else {
			_ = to.Commit()
		}
	}()

	edbInfo = new(EdbInfo)
	edbInfo.Source = obj.GetSource()
	edbInfo.SourceName = obj.GetSourceName()
	edbInfo.EdbCode = edbCode
	edbInfo.EdbName = req.EdbName
	edbInfo.EdbNameSource = req.EdbName
	edbInfo.Frequency = req.Frequency
	edbInfo.Unit = req.Unit
	edbInfo.ClassifyId = req.ClassifyId
	edbInfo.SysUserId = params.SysUserId
	edbInfo.SysUserRealName = params.SysUserRealName
	edbInfo.CreateTime = time.Now()
	edbInfo.ModifyTime = time.Now()
	edbInfo.UniqueCode = params.UniqueCode
	edbInfo.CalculateFormula = req.Formula
	edbInfo.EdbType = obj.GetEdbType()
	newEdbInfoId, tmpErr := to.Insert(edbInfo)
	if tmpErr != nil {
		err = tmpErr
		return
	}
	edbInfo.EdbInfoId = int(newEdbInfoId)

	//关联关系
	{
		calculateMappingItem := new(EdbInfoCalculateMapping)
		calculateMappingItem.CreateTime = time.Now()
		calculateMappingItem.ModifyTime = time.Now()
		calculateMappingItem.Sort = 1
		calculateMappingItem.EdbCode = edbCode
		calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
		calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
		calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
		calculateMappingItem.FromSource = fromEdbInfo.Source
		calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
		calculateMappingItem.FromTag = ""
		calculateMappingItem.Source = edbInfo.Source
		calculateMappingItem.SourceName = edbInfo.SourceName
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//计算数据
	err = obj.refresh(to, edbInfo.EdbInfoId, edbInfo.Source, fromEdbInfo, edbInfo.EdbCode)

	return
}

// Edit 编辑
func (obj Ljzzj) Edit(params EditCalculateBatchParams) (err error, errMsg string) {
	req := params.Req
	edbInfo := params.EdbInfo
	fromEdbInfo := params.FromEdbInfo
	if fromEdbInfo.Frequency != "季度" {
		errMsg = "请选择季度指标"
		err = errors.New(errMsg)
		return
	}

	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
			fmt.Println(reflect.TypeOf(obj).Name(), "；Edit,Err:"+err.Error())
		} else {
			_ = to.Commit()
		}
	}()
	tableName := GetEdbDataTableName(edbInfo.Source)

	//修改指标信息
	edbInfo.EdbName = req.EdbName
	edbInfo.EdbNameSource = req.EdbName
	edbInfo.Frequency = req.Frequency
	edbInfo.Unit = req.Unit
	edbInfo.ClassifyId = req.ClassifyId
	edbInfo.ModifyTime = time.Now()
	_, err = to.Update(edbInfo, "EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "ModifyTime")
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
	existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count > 0 { // 指标未被替换，无需重新计算
		return
	}

	//删除，计算指标关联的，基础指标的关联关系
	sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
	_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
	if err != nil {
		return
	}

	//清空原有数据
	sql = ` DELETE FROM ` + tableName + ` WHERE edb_info_id = ? `
	_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
	if err != nil {
		return
	}

	//关联关系
	{
		calculateMappingItem := &EdbInfoCalculateMapping{
			EdbInfoCalculateMappingId: 0,
			EdbInfoId:                 edbInfo.EdbInfoId,
			Source:                    obj.GetSource(),
			SourceName:                obj.GetSourceName(),
			EdbCode:                   edbInfo.EdbCode,
			FromEdbInfoId:             fromEdbInfo.EdbInfoId,
			FromEdbCode:               fromEdbInfo.EdbCode,
			FromEdbName:               fromEdbInfo.EdbName,
			FromSource:                fromEdbInfo.Source,
			FromSourceName:            fromEdbInfo.SourceName,
			FromTag:                   "",
			Sort:                      1,
			CreateTime:                time.Now(),
			ModifyTime:                time.Now(),
		}
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//计算数据
	err = obj.refresh(to, edbInfo.EdbInfoId, edbInfo.Source, fromEdbInfo, edbInfo.EdbCode)

	return
}

// Refresh 刷新
func (obj Ljzzj) Refresh(params RefreshParams) (err error, errMsg string) {
	calculateMapping, err := GetEdbInfoCalculateMappingDetail(params.EdbInfo.EdbInfoId)
	if err != nil {
		errMsg = "GetEdbInfoCalculateLjzzyDetail Err:" + err.Error()
		return
	}
	fromEdbInfo, err := GetEdbInfoById(calculateMapping.FromEdbInfoId)
	if err != nil {
		errMsg = "GetEdbInfoById Err:" + err.Error()
		return
	}

	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
			fmt.Println(reflect.TypeOf(obj).Name(), "；Refresh,Err:"+err.Error())
		} else {
			_ = to.Commit()
		}
	}()

	// 计算数据
	err = obj.refresh(to, params.EdbInfo.EdbInfoId, params.EdbInfo.Source, fromEdbInfo, params.EdbInfo.EdbCode)

	return
}

// GetSource 获取来源编码id
func (obj Ljzzj) GetSource() int {
	return utils.DATA_SOURCE_CALCULATE_LJZZJ
}

// GetSourceName 获取来源名称
func (obj Ljzzj) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_CALCULATE_LJZZJ
}

// GetEdbType 获取指标类型
func (obj Ljzzj) GetEdbType() int {
	return utils.CALCULATE_EDB_TYPE
}

func (obj Ljzzj) refresh(to orm.TxOrmer, edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode string) (err error) {
	dataTableName := GetEdbDataTableName(source)
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	// 获取来源指标的数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)
	dataList, err := GetEdbDataListAllByTo(to, condition, pars, fromEdbInfo.Source, 1)
	if err != nil {
		return err
	}

	yearMap := make(map[int]map[int]*EdbInfoSearchData)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		item := dataList[i]
		//日其中获取年
		itemDate, err := time.ParseInLocation(utils.FormatDate, item.DataTime, time.Local)
		if err != nil {
			return err
		}
		year := itemDate.Year()
		quarter := utils.MonthQuarterMap[int(itemDate.Month())]
		if quarterMap, yok := yearMap[year]; yok {
			quarterMap[quarter] = item
			yearMap[year] = quarterMap
		} else {
			quarterMap = make(map[int]*EdbInfoSearchData)
			quarterMap[quarter] = item
			yearMap[year] = quarterMap
		}
	}

	addSql := ` INSERT INTO ` + dataTableName + `(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	//获取指标所有数据
	existDataList := make([]*EdbData, 0)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	dataMap := make(map[string]string)
	for _, v := range existDataList {
		dataMap[v.DataTime] = v.Value
	}
	existDataMap := make(map[string]string)
	for yk, yv := range yearMap {
		_, oneQuarterOk := yv[1]
		_, twoQuarterOk := yv[2]
		if !oneQuarterOk && !twoQuarterOk {
			continue
		}
		for i := 1; i <= 4; i++ {
			fmt.Println(yk, i, yv[i])
			dataCurrentItem := yv[i]
			var date string
			var val float64
			if dataCurrentItem != nil {
				if i == 1 || i == 2 {
					if _, mok := yv[1]; mok { //第1个季度有值
						if i == 1 {
							date = dataCurrentItem.DataTime
							val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
						}
						if i == 2 {
							dataOneItem := yv[1]
							date = dataCurrentItem.DataTime
							twoQuarter := decimal.NewFromFloat(dataCurrentItem.Value)
							oneQuarter := decimal.NewFromFloat(dataOneItem.Value)
							val, _ = twoQuarter.Sub(oneQuarter).Float64()
						}
					} else { //第1个季度无值
						dataTwoItem := yv[2]
						if i == 1 {
							date = strconv.Itoa(yk) + "-03-31"
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
						if i == 2 {
							//第1个季度无值：第1个季度=第2个季度/2
							{
								date = strconv.Itoa(yk) + "-03-31"
								a := decimal.NewFromFloat(dataTwoItem.Value)
								b := decimal.NewFromFloat(2.0)
								val, _ = a.Div(b).Float64()

								tmpSql, newAdd, tmpErr := obj.calculate(edbInfoId, date, edbInfoIdStr, edbCode, dataTableName, addSql, val, dataMap, existDataMap, to)
								if !isAdd {
									isAdd = newAdd
								}
								addSql = tmpSql
								if tmpErr != nil {
									return tmpErr
								}
							}
							//end 第1个季度无值

							date = dataCurrentItem.DataTime
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
					}
				} else {
					dataPreItem := yv[i-1]
					if dataCurrentItem != nil && dataPreItem != nil {
						date = dataCurrentItem.DataTime
						//val =  dataCurrentItem.Value - dataPreItem.Value
						a := decimal.NewFromFloat(dataCurrentItem.Value)
						b := decimal.NewFromFloat(dataPreItem.Value)
						val, _ = a.Sub(b).Float64()
					}
				}
			}
			if date != "" {
				tmpSql, newAdd, tmpErr := obj.calculate(edbInfoId, date, edbInfoIdStr, edbCode, dataTableName, addSql, val, dataMap, existDataMap, to)
				if !isAdd {
					isAdd = newAdd
				}
				addSql = tmpSql
				if tmpErr != nil {
					return tmpErr
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			fmt.Println(reflect.TypeOf(obj).Name(), " add data Err", err.Error())
			return
		}
	}
	return
}

func (obj Ljzzj) calculate(edbInfoId int, date, edbInfoIdStr, edbCode, dataTableName, addSql string, val float64, dataMap, existDataMap map[string]string, to orm.TxOrmer) (newSql string, isAdd bool, err error) {
	newSql = addSql
	saveValue := utils.SubFloatToString(val, 4)
	//判断数据是否存在
	if existVal, ok := dataMap[date]; !ok {
		dataTime, _ := time.ParseInLocation(utils.FormatDate, date, time.Local)
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, existOk := existDataMap[date]; !existOk {
			newSql += GetAddSql(edbInfoIdStr, edbCode, date, timeStr, saveValue)
			isAdd = true
		}
		existDataMap[date] = date
	} else {
		if existVal != saveValue {
			sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
			sql = fmt.Sprintf(sql, dataTableName)
			_, err = to.Raw(sql, saveValue, edbInfoId, date).Exec()
		}
	}
	return
}
